import { hasFunction, IContext, IParam, isExist, ViewUtil } from '@core';

/**
 * 界面行为参数对象
 *
 * @export
 * @interface IUIActionParams
 */
export interface IUIActionParams {
  /**
   * 上下文参数
   */
  context: IContext;
  /**
   * 视图参数
   */
  viewParams: IParam;
  /**
   * 当前激活数据
   */
  data: IParam;
  /**
   * js事件源对象
   */
  event: MouseEvent;
  /**
   * 当前上下文环境引用
   */
  actionEnvironment: IParam;
}

/**
 * 系统预置界面行为
 *
 * @export
 * @class AppSysAction
 */
export class AppSysAction {
  /**
   * 执行预置界面行为
   *
   * @static
   * @param {IParam} uiAction 界面行为对象
   * @param {IUIActionParams} params 界面行为参数对象
   * @memberof AppSysAction
   */
  public static async execute(uiAction: IParam, params: IUIActionParams) {
    const tag = uiAction.uIActionTag;
    switch (tag) {
      case 'Edit':
        this.edit(params);
        break;
      case 'New':
        this.new(params);
        break;
      case 'Save':
        this.save(params);
        break;
      case 'SaveRow':
        this.saveRow(params);
        break;
      case 'Remove':
        this.remove(params);
        break;
      case 'ToggleRowEdit':
        this.toggleRowEdit(params);
        break;
      case 'NewRow':
        this.newRow(params);
        break;
      default:
        console.log(`未支持${tag}`);
    }
  }

  /**
   * 编辑
   *
   * @static
   * @param params 界面行为参数对象
   * @return {*} 
   */
  public static edit(params: IUIActionParams) {
    const { actionEnvironment } = params;
    if (!actionEnvironment?.state?.viewLogics?.openData?.openDataViewName) {
      return;
    }
    const viewName = actionEnvironment?.state.viewLogics.openData.openDataViewName;
    const view = App.getViewInfo(viewName);
    if (!view) {
      return;
    }
    ViewUtil.openData(view, params);
  }

  /**
   * 新建
   *
   * @param params 界面行为参数对象
   * @return {*}
   */
  public static new(params: IUIActionParams) {
    const { actionEnvironment } = params;
    if (!actionEnvironment?.state?.viewLogics?.newData?.newDataViewName) {
      return;
    }
    const viewName = actionEnvironment?.state.viewLogics.newData.newDataViewName;
    const view = App.getViewInfo(viewName);
    if (!view) {
      return;
    }
    ViewUtil.newData(view, params);
  }

  /**
   * 行编辑
   * 
   * @param params 界面行为参数对象
   * @return {*}
   */
  public static toggleRowEdit(params: IUIActionParams) {
    const { actionEnvironment } = params;
    if(isExist(actionEnvironment.state.rowEditState)){
      actionEnvironment.state.rowEditState = !actionEnvironment.state.rowEditState;
    }
  }

  /**
   * 新建行
   * 
   * @param params 界面行为参数对象
   * @return {*}
   */
  public static newRow(params: IUIActionParams) {
    const { actionEnvironment } = params;
    // 视图里获取多数据部件
    if(hasFunction(actionEnvironment.xDataControl, "newRow")){
      actionEnvironment.xDataControl.newRow();
    }else if(isExist(actionEnvironment.newRow)){
      actionEnvironment.newRow();
    }
  }

  /**
   * 保存
   * 
   * @param params 界面行为参数对象
   * @return {*}
   */
  public static save(params: IUIActionParams) {
    const { actionEnvironment } = params;
    // 视图里获取多数据部件
    if(hasFunction(actionEnvironment.xDataControl, "save")){
      actionEnvironment.xDataControl.save();
    }else if(isExist(actionEnvironment.save)){
      actionEnvironment.save();
    }
  }

  /**
   * 保存行
   * 
   * @param params 界面行为参数对象
   * @return {*}
   */
  public static saveRow(params: IUIActionParams) {
    const { actionEnvironment } = params;
    // 视图里获取多数据部件
    if(hasFunction(actionEnvironment.xDataControl, "save")){
      actionEnvironment.xDataControl.save();
    }else if(isExist(actionEnvironment.save)){
      actionEnvironment.save();
    }
  }

  /**
   * 删除
   * 
   * @param params 界面行为参数对象
   * @return {*}
   */
  public static remove(params: IUIActionParams) {
    const { actionEnvironment } = params;
    // 视图里获取多数据部件
    if(hasFunction(actionEnvironment.xDataControl, "remove")){
      actionEnvironment.xDataControl.remove();
    }else if(isExist(actionEnvironment.remove)){
      actionEnvironment.remove();
    }
  }
}
